module("veraTemplate", package.seeall)
--
-- Vera Template Library
-- Copyright (C) 2012 Richard T. Schaefer
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
--
----------------------------------------------------

-- Private ------------------------------------------------------------------------
local Version = "1.0.1"
local TemplatePattern =  "%b{}"
local TemplateClasses = {}

-- p1 = Class, p2 = SunFunction, p3 = Junk upto ( or |) p4 = rest
local ParseTemplateStruncture  = "^[%s]*([%w]+)([%.%w]*)([^%(%|]*)([%s%S]*)$"

local function ExpandATemplate(Dict, Text, PipeArg)  
   --Log("ExpandATemplate",  "Dict:" .. Dict .. " Text:" .. Text .. " PipeArg:" .. (PipeArg or "nil"))
   local s,f,Class,SubFunction,p3,p4 = string.find(Text, ParseTemplateStruncture)
   local Args
   local Rest
   local exp
   if (p4 and string.find(p4,"^%(")) then
      s,f,Args = string.find(p4, "(%b())[%s]*")
      Args = string.sub(Args, 2, -2)
      Rest = string.sub(p4, f+1)
      --Log("ExpandATemplate",  "Rest:" .. (Rest or "nil"))
   end
   if (SubFunction and (string.len(SubFunction) > 0)) then
      SubFunction = string.sub(SubFunction, 2)
   end
   
   local DictVal = TemplateClasses["Template"]
   local t = DictVal[string.lower(Class)]
   if (t) then
      if (type(t) == "string") then
         exp = t
      else
         exp = t.expand(string.lower(SubFunction), Args, PipeArg)
      end
      --Log("ExpandATemplate",  "Dict:" .. Dict .. " Class:" .. Class .. " SubFunction:" .. (SubFunction or "nil") .. " Args:" .. (Args or "nil")  .. " Result:" .. (exp or "nil"))
   else
      -- Pass thru unknow stuff
      return
   end
   
   if (Rest and string.len(Rest) > 0) then
      -- We are Chained ... Parse the next component
      exp = ExpandATemplate(Dict, string.sub(Rest,2),  exp)
   end
   return exp
end

-- Public ------------------------------------------------------------------------
-- Absent  Defines the behaviour of a template that does not expand  (0: Untouched, 1: Remove Braces, 2: Empty String)
Absent = 0

function expand(txt, Dict)
   local Dict  = Dict or "Template"
   return(string.gsub(txt, TemplatePattern, 
                      function(Text) 
                         local exp = ExpandATemplate(Dict, expand(string.sub(Text,2,-2)))
                         --Log("Expand:",  tostring(exp) .. " Absent:" .. tostring(Absent))
                         if (exp) then
                            exp = tostring(exp)
                         else
                            if (Absent == 1) then
                               exp = string.sub(Text,2,-2)
                            elseif (Absent == 2) then
                               exp = "";
                            end
                         end
                         return exp
                      end))
end


function register(Template, Definition, Dict)
   Dict = Dict or "Template"
   local DictTab = TemplateClasses[Dict]
   if (not DictTab) then
      DictTab = {}
   end
   if (type(Template) == "string" )  then
      --print("Register String: " .. Template .. " Dict:"  .. Dict)
      DictTab[string.lower(Template)] = Definition
   else
      --print("Register Class: " .. Template.templateclass)
      DictTab[string.lower(Template.templateclass)] = Template
   end
   TemplateClasses[Dict] = DictTab
end

function lookup(Template, Dict)
   Dict = Dict or "Template"
   local DictTab = TemplateClasses[Dict]
   return DictTab[string.lower(Template)]
end

function classes()
   local s = ""
   local DictKey
   local DictVal
   for DictKey, DictVal in pairs(TemplateClasses) do
      local ClassKey
      local ClassVal
      for ClassKey, ClassVal in pairs(DictVal) do
         s  = s .. DictKey .. ": {" .. ClassKey
         if (type(ClassVal) == "string") then
            s = s .. "} = \"" .. ClassVal .. "\""
         else
            s = s .. "} is a functional template"
         end
         s = s .. "\n"
      end
   end
   return s
end

function version()
   return(Version)
end

-- Standard Classes

register(require("veraTemplateString"));
register(require("veraTemplateDateTime"));
register(require("veraTemplateDevice"));
register(require("veraTemplateSystem"));
register(require("veraTemplateNumber"));
register(require("veraTemplateChoose"));
