module("veraTemplateDevice", package.seeall)
--
-- Vera Template Library
-- Copyright (C) 2012 Richard T. Schaefer
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
--
----------------------------------------------------

-- Private ------------------------------------------------------------------------

local function RoomProperty(dev, Args) 
   --print ("Device.RoomProperty: " .. dev .. " Args:" .. Args)
   local room
   local res
   if (luup) then
      room = luup.devices[tonumber(dev)].room_num
   else
      room = "luup.devices[" .. dev .. "].room_num"
   end
   if (string.lower(Args) == 'name') then
      if (luup) then
	 res = luup.rooms[room]
      else
	 res = "luup.rooms[" .. room .. "]"
      end
   else
      res = room
   end
   return res
end

local function SrvcProperty(dev, Srvc, Args)
   --print ("Device.SrvcProperty: " .. dev .. " Srvc:" .. Srvc .. " Args:" .. Args)
   local res
   if (luup) then
      res = luup.variable_get(Srvc, Args, tonumber(dev))
   else
      res = "luup.variable_get(" .. Srvc .. ", " .. Args .. ", " ..  dev .. ")"
   end
   return res
end

local function PropertyRead(dev, Args)
   --print("Device.PropertyRead:" .. dev .. " Args:" .. Args)
   local s, f, srvc  = string.find(Args, "^service(%b[])")
   if (srvc) then
      res = SrvcProperty(dev, string.sub(srvc,2,-2), string.sub(Args, string.len(srvc) + 9))
   else
      local s, f, field = string.find(Args, "^[%s]*([%w]*)[%.%s%p]*")
      Args =  string.sub(Args, f + 1)
      local lfield = string.lower(field)
      if (lfield == 'room') then
	 res = RoomProperty(dev,  Args)
      else
	 -- Check if Field is a service alias
	 local t = require("veraTemplate")
	 local s = t.lookup(field, "Service")
	 if (s) then
	    res = PropertyRead(dev, s .. "."  .. Args)
	 else
	    --- Must be a device property
	    if (luup) then
	       res = luup.devices[tonumber(dev)][field]
	    else
	       res =  "luup.devices[" .. dev .. "]." ..  field
	    end
	 end
      end
   end
   return  res
end


local function DevRead(dev, Args)
   -- print("Device.DevRead:" .. dev .. " Args:" .. Args);
   local s, f = string.find(Args,  "^parent[%.%s%p]*")
   local res
   if (f) then
      if (luup) then
	 dev = luup.devices[dev].parent
      else
	 dev = "luup.devices[" .. dev .. "].parent"
      end
      res = DevRead(dev, string.sub(Args, f+1))
   else
      if (string.len(Args) > 0) then
	 res = PropertyRead(dev, Args)
      else
	 res = dev
      end
   end
   return res
end


-- Public ------------------------------------------------------------------------
templateclass="Device"
device = 0

-- Should never pipe into Device
function expand(SubFunction, Args, PipeArg)
   Args = Args or ""
   local s, f, dev, rest = string.find(Args, "^(%b[])[%s%.]*([%s%S]*)")
   if (dev) then
      dev = string.sub(dev, 2, -2)
   else 
      dev = tostring(device)
      rest = Args
   end
   local devnum = tostring(tonumber(dev))
   if (dev ~= devnum) then
      if (luup) then
	 for i,d in ipairs(luup.devices) do
	    if (d.description == dev) then
	       dev = i
	       break
	    end
	 end
      else
	 dev = "Lookup(" .. dev .. ")"
      end
   end
   local res = dev
   if (string.len(rest) > 0) then
      res = DevRead(dev, rest)
   end
   --Log("veraTemplateDevice", "SubFunction:" .. (SubFunction or "nil") .. " Args:" .. (Args or "nil") .." rest:" .. (rest or "nil") .." res:" .. (res or "nil"))
   return res
end
