module("veraTemplateTest", package.seeall)
--
-- Vera Template Library
-- Copyright (C) 2012 Richard T. Schaefer
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
--
----------------------------------------------------

function test()
   local t = require("veraTemplate")
   t.Absent = 2

   require("veraTemplateDefinitions")
   if (not pcall(require, "veraUserTemplateDefinitions")) then
      print("Sorry Can't find: veraUserTemplateDefinitions.lua")
   end

   -- Set the Default Device
   local d = require("veraTemplateDevice")
   d.device = 20

   print("Parse Classes:")
   print(t.classes())

   -- Various Expansions

   print(t.expand("Test Number 3.14159  !"))
   print(t.expand("Test Number String Format {String.format(%20.2e,3.14159)}  !"))
   print(t.expand("Test Number with Pipe to String.Format1 {Number(3.14159) | String(%20.2e)}  !"))
   print(t.expand("Test Number {Number(3.14159,%20.2e)}  !"))

   print(t.expand("Test DateTime CurrentTime {dAtEtIme(currenttime)}  !"))
   print(t.expand("Test DateTime Format CurrentTime {DateTime(currenttime,%A %B %d %Y %H:%M)}  !"))
   print(t.expand("Test DateTime Format Past Time {DateTime(1111100000,{DateTimeFormat})}  !"))

   print(t.expand("Test String Upper {String.upper(AbCdEfG)}  !"))
   print(t.expand("Test String Lower {String.lower(AbCdEfG)}  !"))
   print(t.expand("Test String RFill {String.rfill(AbCdEfG   ,R,20)}  !"))
   print(t.expand("Test String RFill {String.rfill(AbCdEfG   ,R,2)}  !"))
   print(t.expand("Test String LFill {String.lfill(   AbCdEfG,L,20)}  !"))
   print(t.expand("Test String LFill {String.lfill(   AbCdEfG,L,2)}  !"))
   print(t.expand("Test String Truncate {String.truncate(AbCdEfG,3)}  !"))
   print(t.expand("Test String Format {String.format(%20s %-20s,String 1,String 2)}  !"))
   print(t.expand("Test String Format with Pipe {String.format(%20s %-20s,String 1,String 2) | String.lfill(L,1) | String.rfill(R,1)}  !"))
   print(t.expand("Test String Format with NewLines {String.format(\n%s  !\n%20s  !\n%-20s,Text Line 1,Text Line2,Text Line 3)}  !"))

   print(t.expand("Test System {System(city)}  !"))

   print(t.expand("Test Choose 0 {Choose(0, {LockOptions})} !"))
   print(t.expand("Test Choose 1 {Choose(1, {LockOptions})} !"))
   print(t.expand("Test Choose 2 {Choose(2, {LockOptions})} !"))
   
   print(t.expand("Test Device {device}  !"))
   print(t.expand("Test Device {device(description)}  !"))
   print(t.expand("Test Device {device(room)}  !"))
   print(t.expand("Test Device {device(room.name)}  !"))
   print(t.expand("Test Device {device(service[foobar].Status)}  !"))
   print(t.expand("Test Device {device(service[urn:upnp-org:serviceId:SwitchPower1].Status)}  !"))
   print(t.expand("Test Device {device(switch.Status)}  !"))

   print(t.expand("Test Device {device([5])}  !"))
   print(t.expand("Test Device {device([5].description)}  !"))
   print(t.expand("Test Device {device([5]room)}  !"))
   print(t.expand("Test Device {device([5].room.name)}  !"))
   print(t.expand("Test Device {device([5].service[foobar].Status)}  !"))
   print(t.expand("Test Device {device([5].service[urn:upnp-org:serviceId:SwitchPower1].Status)}  !"))
   print(t.expand("Test Device {device([5].Switch.Status)}  !"))

   print(t.expand("Test Device {device([Office Light])}  !"))
   print(t.expand("Test Device {device([Office Light].description)}  !"))
   print(t.expand("Test Device {device([Office Light].room)}  !"))
   print(t.expand("Test Device {device([Office Light].room.name)}  !"))
   print(t.expand("Test Device {device([Office Light].service[foobar].Status)}  !"))
   print(t.expand("Test Device {device([Office Light].service[urn:upnp-org:serviceId:SwitchPower1].Status)}  !"))
   print(t.expand("Test Device {device([Office Light].Switch.Status)}  !"))

   print(t.expand("Test Device {device([9].parent)}  !"))
   print(t.expand("Test Device {device([9].parent.description)}  !"))
   print(t.expand("Test Device {device([9].parent.room)}  !"))
   print(t.expand("Test Device {device([9].parent.room.name)}  !"))
   print(t.expand("Test Device {device([9].parent.service[foobar].Status)}  !"))
   print(t.expand("Test Device {device([9].parent.service[urn:upnp-org:serviceId:SwitchPower1].Status)}  !"))
   print(t.expand("Test Device {device([9].parent.Switch.Status)}  !"))

   print(t.expand("Test Device {device([9].service[urn:upnp-org:serviceId:SwitchPower1].Status)} !"))
   print(t.expand("Test Device {device([9].service[urn:upnp-org:serviceId:SwitchPower1].Status) | Choose(0=Off,1=On,Other=Unknown)}  !"))
   print(t.expand("Test Device {device([9].service[urn:upnp-org:serviceId:SwitchPower1].Status) | Choose({BinarySwitchOptions})}  !"))
   
   print(t.expand("Test Device {device([9].Switch.Status)}  !"))
   print(t.expand("Test Device {device([9].Switch.Status) | Choose(0=Off,1=On,Other=Unknown)}  !"))
   print(t.expand("Test Device {device([9].Switch.Status) | Choose({BinarySwitchOptions})}  !"))

   print(t.expand("Test Device & DateTime Choose Nested Template: {device([9].description)} last update on {DateTime({device([9].HomeAutomation.LastUpdate)}, !{DateTimeFormat})} GMT !"))
   print(t.expand("Test Device & DateTime Switch Pipe through Choose: {device([9].description)} last update on {device([9].HomeAutomation.LastUpdate) |  DateTime(!{DateTimeFormat})} GMT !"))
end
